library resource_widget;

import 'package:flutter/material.dart';
import 'package:resource_result/resource_result.dart';

class _ResourceBuilder<T> extends StatelessWidget {
  final AsyncSnapshot<Resource<T>> resourceSnapshot;
  final Widget Function(BuildContext context, T data) success;
  final WidgetBuilder loadingIndicator;

  /// Defaults to [handleErrorAsText]
  Widget? Function(BuildContext context, Error error)? handleError;

  _ResourceBuilder(
      {required this.resourceSnapshot,
      required this.success,
      required this.loadingIndicator,
      this.handleError,
      Key? key})
      : super(key: key);

  Text handleErrorAsText(_, Error error) => Text("Error: ${error.message}",
      style: const TextStyle(color: Colors.red, backgroundColor: Colors.white));

  @override
  Widget build(BuildContext context) {
    handleError ??= handleErrorAsText;
    if (resourceSnapshot.connectionState == ConnectionState.waiting) {
      return loadingIndicator(context);
    }
    if (resourceSnapshot.hasData) {
      final resource = resourceSnapshot.data!;
      if (resource is Success) return success(context, resource.data!);
      if (resource is Loading) return loadingIndicator(context);
      final errorWidget =
          handleError!(context, (resourceSnapshot.data?.error)!);
      if (errorWidget != null) return errorWidget;
    }
    final errorWidget = handleError!(
        context, Error("Error in loading future ${resourceSnapshot.error}"));
    if (errorWidget != null) return errorWidget;
    return const SizedBox();
  }
}

/// Uses flutter widget [FutureBuilder] to handle a [Future]<[Resource]<T>>
/// and handle the different states of it differently
/// through the passed functions [success], [loadingIndicator], [handleError]
class ResourceFutureBuilder<T> extends StatelessWidget {
  final Future<Resource<T>>? future;
  final Resource<T>? initialData;
  final Widget Function(BuildContext context, T data) success;
  final WidgetBuilder loadingIndicator;
  final Widget? Function(BuildContext context, Error error)? handleError;

  const ResourceFutureBuilder(
      {required this.future,
      this.initialData,
      required this.success,
      required this.loadingIndicator,
      this.handleError,
      Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<Resource<T>>(
        key: key,
        future: future,
        initialData: initialData ?? Loading(null),
        builder: (context, snapshot) {
          return _ResourceBuilder(
              resourceSnapshot: snapshot,
              success: success,
              loadingIndicator: loadingIndicator,
              handleError: handleError);
        });
  }
}

/// Uses flutter widget [StreamBuilder] to handle a [Stream]<[Resource]<T>>
/// and handle the different states of it differently
/// through the passed functions [success], [loadingIndicator], [handleError]
class ResourceStreamBuilder<T> extends StatelessWidget {
  final Stream<Resource<T>>? stream;
  final Resource<T>? initialData;
  final Widget Function(BuildContext context, T data) success;
  final WidgetBuilder loadingIndicator;
  final Widget? Function(BuildContext context, Error error)? handleError;

  const ResourceStreamBuilder(
      {required this.stream,
      this.initialData,
      required this.success,
      required this.loadingIndicator,
      this.handleError,
      Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<Resource<T>>(
        key: key,
        stream: stream,
        initialData: initialData ?? Loading(null),
        builder: (context, snapshot) {
          return _ResourceBuilder(
              resourceSnapshot: snapshot,
              success: success,
              loadingIndicator: loadingIndicator,
              handleError: handleError);
        });
  }
}
